@@ -171,7 +171,6 @@ soup_message_cleanup (SoupMessage *req)
 	g_return_if_fail (req != NULL);
 
 	if (req->connection && 
-	    soup_connection_is_keep_alive (req->connection) &&
 	    req->priv->read_tag &&
 	    req->status == SOUP_STATUS_READING_RESPONSE) {
 		soup_transfer_read_set_callbacks (req->priv->read_tag,
@@ -774,6 +773,10 @@ authorize_handler (SoupMessage *msg, gboolean proxy)
 	SoupContext *ctx;
 	const SoupUri *uri;
 
+	if (msg->connection->auth &&
+	    msg->connection->auth->status == SOUP_AUTH_STATUS_SUCCESSFUL)
+		goto THROW_CANT_AUTHENTICATE;
+
 	ctx = proxy ? soup_get_proxy () : msg->context;
 	uri = soup_context_get_uri (ctx);
 
@@ -856,9 +859,9 @@ authorize_handler (SoupMessage *msg, gboolean proxy)
 	soup_auth_initialize (auth, uri);
 
 	if (auth->type == SOUP_AUTH_TYPE_NTLM) {
-		SoupAuth *old_auth = soup_auth_lookup (ctx);
+		SoupAuth *old_auth = msg->connection->auth;
 
-		if (old_auth && auth != old_auth) 
+		if (old_auth)
 			soup_auth_free (old_auth);
 		msg->connection->auth = auth;
 	} else
@@ -1000,14 +1003,14 @@ static SoupHandlerData global_handlers [] = {
 		{ 407 }
 	},
 	/*
-	 * Handle response codes 2xx for validating auths.
+	 * Handle Authorization-Info headers for validating auths.
 	 */
 	{
 		SOUP_HANDLER_PRE_BODY,
 		(SoupCallbackFn) validate_authorize_handler,
 		NULL,
-		RESPONSE_ERROR_CLASS_HANDLER,
-		{ SOUP_ERROR_CLASS_SUCCESS }
+		RESPONSE_HEADER_HANDLER,
+		{ (guint) "Authentication-Info" }
 	},
 	{ 0 }
 };
